#include "UnitTest++.h"


#include <sys/stat.h> // use to create a directory in ubuntu system

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>

#include "./src/Models/HelperFunction.hpp"
#include "./src/Models/FNVModel.hpp"
#include "./src/Models/SolveNonlinearEquation.hpp"



namespace
{

///==================================================================================================
/// Tests for Manuscript 2
///===================================================================================================
TEST(ViscoelasticStressRelaxation)
{
    std::cout << "start test: ViscoelasticStressRelaxation " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction HObj;
    FNVModel fnvObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/ViscoelasticStressRelaxation", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/ViscoelasticStressRelaxation/Time_F_B_Be_Soe_S.txt");



    //===========================================================================================
    // time discritization
    const double dt = 0.01;// time increment for mechanics, unit: sec
    const double time_loading = 0.1;// time for stretching, unit: sec
    const double time_holding = 5.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int num_step_time = time_simulation / dt;

    const double FinalStretch = 1.2; //




    //===========================================================================================
    // material
    fnvObj.SetNeoHookeanMaterialParams(1.0,1.0,2.0);



    //===========================================================================================
    // Initialization of tensors
    std::vector< std::vector<double> > F        = HObj.Identity(3);
    std::vector< std::vector<double> > F_old    = HObj.Identity(3);
    std::vector< std::vector<double> > Be       = HObj.Identity(3);
    std::vector< std::vector<double> > Soe(3,std::vector<double> (3,0.0));




    //===========================================================================================
    // time looping
    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        if (time <= time_loading)
        {
            F[0][0]=1.0+(FinalStretch-1.0)*(time/time_loading);
        }
        else
        {
            F[0][0]= FinalStretch;
        }
        F[1][1]=1/sqrt(F[0][0]); F[2][2] = F[1][1];


        std::vector< std::vector<double> > PassiveStress(3,std::vector<double> (3,0.0));
        fnvObj.CalculateStress(F,F_old,Soe,Be,dt,PassiveStress,fnvObj.CauchyStress,fnvObj.UniaxialLoading);

        std::vector< std::vector<double> > transF = HObj.Transpose(F);
        std::vector< std::vector<double> > B = HObj.Multiply(F,transF);

        outputFile1 << time << "\t" << F[0][0] << "\t" << B[0][0] << "\t" << Be[0][0] << "\t" <<
                        Soe[0][0] << "\t" << PassiveStress[0][0] << "\t" << std::endl;

    }// end of mechanics time looping


    //===========================================================================================
    // cleaning
    outputFile1.close();

}

TEST(ViscoelasticCreeping)
{
    std::cout << "start test: ViscoelasticCreeping " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction HObj;
    FNVModel fnvObj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/ViscoelasticCreeping", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/ViscoelasticCreeping/Time_F_B_Be_Soe_S.txt");



    //===========================================================================================
    // time discritization
    const double dt = 0.01;// time increment for mechanics, unit: sec
    const double time_loading = 0.1;// time for stretching, unit: sec
    const double time_holding = 5.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int num_step_time = time_simulation / dt;

    const double FinalStress = 1.213; //




    //===========================================================================================
    // material
    double Ce = 1.0;
    double Coe = 1.0;
    double eta = 2.0;
    fnvObj.SetNeoHookeanMaterialParams(Ce,Coe,eta);



    //===========================================================================================
    // Initialization of tensors
    std::vector< std::vector<double> > F        = HObj.Identity(3);
    std::vector< std::vector<double> > F_old    = HObj.Identity(3);
    std::vector< std::vector<double> > Be       = HObj.Identity(3);
    std::vector< std::vector<double> > Soe(3,std::vector<double> (3,0.0));




    //===========================================================================================
    // time looping
    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        if (time <= time_loading)
        {
            double intStress = FinalStress*(time/time_loading);

            sneObj.SetParameterValues(Ce,Coe,F[0][0],sqrt(Be[0][0]),dt,eta,intStress);
            sneObj.SetInitialGuess(F[0][0]);
            double lambda = sneObj.SolveEquation();

            F[0][0]=lambda;
        }
        else
        {
            sneObj.SetParameterValues(Ce,Coe,F[0][0],sqrt(Be[0][0]),dt,eta,FinalStress);
            sneObj.SetInitialGuess(F[0][0]);
            double lambda = sneObj.SolveEquation();

            F[0][0]=lambda;
        }
        F[1][1]=1/sqrt(F[0][0]); F[2][2] = F[1][1];


        std::vector< std::vector<double> > PassiveStress(3,std::vector<double> (3,0.0));
        fnvObj.CalculateStress(F,F_old,Soe,Be,dt,PassiveStress,fnvObj.CauchyStress,fnvObj.UniaxialLoading);

        std::vector< std::vector<double> > transF = HObj.Transpose(F);
        std::vector< std::vector<double> > B = HObj.Multiply(F,transF);



        outputFile1 << time << "\t" << F[0][0] << "\t" << B[0][0] << "\t" << Be[0][0] << "\t" <<
                        Soe[0][0] << "\t" << PassiveStress[0][0] << "\t" << std::endl;

    }// end of mechanics time looping


    //===========================================================================================
    // cleaning
    outputFile1.close();

}

TEST(CylinderInflationStressRelaxation)
{
    std::cout << "start test: CylinderInflationStressRelaxation " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/CylinderInflationStressRelaxation", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/CylinderInflationStressRelaxation/RadialStress.txt");
    std::ofstream outputFile2;
    outputFile2.open("./output/CylinderInflationStressRelaxation/TangentialStress.txt");



    //===========================================================================================
    // time discritization
    const double dt              = 1.0;// time increment for mechanics, unit: sec
    const double time_loading    = 10.0;// time for stretching, unit: sec
    const double time_holding    = 0.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;


    //===========================================================================================
    // space discritization
    const double A_ref = 1.0;
    const double B_ref = 2.0;
    const double a_cur = 1.5;
    const double lambda_axialstretch = 1.0;

    const double dR_ref          = 0.01;// space discritization
    const int    num_step_space  = (B_ref-A_ref)/dR_ref;
    const double Inc_a_with_time = (a_cur - A_ref) / (time_loading/dt);





    //===========================================================================================
    // material
    fnvObj.SetNeoHookeanMaterialParams(1.0,1.0,2.0);




    // vector & tensor initialization ==================================================================================
    // initialization of tensors
    std::vector< std::vector<double> > Imatrix = Hobj.Identity(3);// Identity matrix
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix
    std::vector< std::vector< std::vector<double> > > F_old_global (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global    (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global   (num_step_space+1,Zmatrix );






    //===========================================================================================
    // time looping
    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        std::vector<double> IntegralFunction (num_step_space+1, 0.0);
        double Area = 0.0;

        std::vector<double> Stress_radial(num_step_space+1, 0.0);
        std::vector<double> Stress_tan   (num_step_space+1, 0.0);
        std::vector<double> Stress_axial (num_step_space+1, 0.0);

        double a_int;
        if (time <= time_loading)
        {
            a_int = A_ref + Inc_a_with_time * step;
        }
        else
        {
            a_int = a_cur;
        }


        //=================================================================================================
        // space looping
        for (int loop2 = 0; loop2 <= num_step_space; loop2++)
        {
            double R_ref = A_ref + dR_ref*loop2;
            double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

            double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
            double lambda_T = r_int/R_ref;

            std::vector< std::vector<double> > F_tensor = Imatrix;
            F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


            std::vector< std::vector<double> > PassiveStress = Zmatrix;
            fnvObj.CalculateStress(F_tensor,F_old_global[loop2],Soe_global[loop2],Be_global[loop2],dt,
                                    PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


            // solve equilibrium equation
            IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
            double StripArea;
            if (R_ref == A_ref)
            {
                StripArea=0.0;
            }
            else
            {
                StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
            }
            Area = Area+StripArea;

            Stress_radial[loop2]= Area;
            Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
            Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

        }// end of space looping, i.e. loop2


        double Hyd_pressure = -1.0*Stress_radial[num_step_space];
        for (int i=0;i<=num_step_space;i++)
        {
            Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
            Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
            Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
        }


        if (time >= 0.0)
        {
            for (unsigned i=0;i<Stress_radial.size(); i++)
            {
                outputFile1 << Stress_radial[i] << "\t";
            }
            outputFile1 << std::endl;

            for (unsigned i=0;i<Stress_tan.size(); i++)
            {
                outputFile2 << Stress_tan[i] << "\t";
            }
            outputFile2 << std::endl;
        }

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();

}

TEST(CylinderInflationCreeping)
{
    std::cout << "start test: CylinderInflationCreeping " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/CylinderInflationCreeping", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/CylinderInflationCreeping/Time_Pressure_aint.txt");
    std::ofstream outputFile2;
    outputFile2.open("./output/CylinderInflationCreeping/Radialstress.txt");
    std::ofstream outputFile3;
    outputFile3.open("./output/CylinderInflationCreeping/Circumstress.txt");


    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_loading    = 0.1;// time for stretching, unit: sec
    const double time_holding    = 6.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;

    double FinalInternalPressure = 0.8565;


    //===========================================================================================
    // space discritization
    const double A_ref = 1.0;
    const double B_ref = 2.0;
    const double lambda_axialstretch = 1.0;
    const double dR_ref          = 0.1;// space discritization
    const int    num_step_space  = (B_ref-A_ref)/dR_ref;

    //===========================================================================================
    // material
    double Ce =1.0;
    double Coe = 1.0;
    double eta = 2.0;
    fnvObj.SetNeoHookeanMaterialParams(Ce,Coe,eta);


    // vector & tensor initialization ==================================================================================
    sneObj.SetGeometryParamsForGoldenSection(A_ref,B_ref,dR_ref,lambda_axialstretch);
    sneObj.SetMaterialParamsForGoldenSection(Ce,Coe,eta);

    // initialization of tensors
    std::vector< std::vector<double> > Imatrix = Hobj.Identity(3);// Identity matrix
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix
    std::vector< std::vector< std::vector<double> > > F_old_global (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global    (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global   (num_step_space+1,Zmatrix );


    //===========================================================================================
    // time looping
    double a_int = A_ref;

    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        double AppliedPressure;
        if (time <= time_loading)
        {
            AppliedPressure = FinalInternalPressure * ( time/time_loading);
        }
        else
        {
            AppliedPressure = FinalInternalPressure;
        }
        a_int = sneObj.FindRootGoldenSection(AppliedPressure,a_int,dt);

        outputFile1 << time << "\t" << AppliedPressure << "\t" << a_int << std::endl;


        if (true)
        {
            std::vector<double> IntegralFunction (num_step_space+1, 0.0);
            double Area = 0.0;

            std::vector<double> Stress_radial(num_step_space+1, 0.0);
            std::vector<double> Stress_tan   (num_step_space+1, 0.0);
            std::vector<double> Stress_axial (num_step_space+1, 0.0);

             //=================================================================================================
            // space looping
            for (int loop2 = 0; loop2 <= num_step_space; loop2++)
            {
                double R_ref = A_ref + dR_ref*loop2;
                double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

                double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
                double lambda_T = r_int/R_ref;

                std::vector< std::vector<double> > F_tensor = Imatrix;
                F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


                std::vector< std::vector<double> > PassiveStress =Zmatrix;
                fnvObj.CalculateStress(F_tensor,F_old_global[loop2],Soe_global[loop2],Be_global[loop2],dt,
                                        PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


                // solve equilibrium equation
                IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
                double StripArea;
                if (R_ref == A_ref)
                {
                    StripArea=0.0;
                }
                else
                {
                    StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
                }
                Area = Area+StripArea;

                Stress_radial[loop2]= Area;
                Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
                Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

            }// end of space looping, i.e. loop2


            double Hyd_pressure = -1.0*Stress_radial[num_step_space];
            for (int i=0;i<=num_step_space;i++)
            {
                Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
                Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
                Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
            }

            assert(isnan(Stress_radial[0]) == false);

            for (unsigned i=0;i<Stress_radial.size(); i++)
            {
                outputFile2 << Stress_radial[i] << "\t";
                outputFile3 << Stress_tan[i] << "\t";
            }
            outputFile2 << std::endl;
            outputFile3 << std::endl;
        }

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();
    outputFile2.close();
    outputFile3.close();

}

TEST(CarnielStressRelaxation)
{
    std::cout << "start test: CarnielStressRelaxation " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/CarnielStressRelaxation", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/CarnielStressRelaxation/RadialStress_5p75.txt");
    std::ofstream outputFile2;
    outputFile2.open("./output/CarnielStressRelaxation/RadialStress_4p0.txt");



    //===========================================================================================
    // time discritization
    double dt              = 0.1;// time increment for mechanics, unit: sec
    double time_loading    = 1.0;// time for stretching, unit: sec
    double time_holding    = 150.0;// time for holding constant stretch
    double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    int    num_step_time   = time_simulation / dt;


    //===========================================================================================
    // material
//    fnvObj.SetNeoHookeanMaterialParams(0.31,7.12,900.5); // these are the optimized params

    std::vector<double> eta(5,1.0);
    eta[0]=600.5;
    eta[1]=0.5;
    fnvObj.SetNeoHookeanMaterialParams(0.31,7.92,eta);



    //===========================================================================================
    // space discritization
    const double A_ref = 1.25;
    const double B_ref = 2.0;
    const double lambda_axialstretch = 1.0;
    const double dR_ref          = 0.01;// space discritization
    const int    num_step_space  = (B_ref-A_ref)/dR_ref;


    // vector & tensor initialization ==================================================================================
    // initialization of tensors
    std::vector< std::vector<double> > Imatrix = Hobj.Identity(3);// Identity matrix
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix
    std::vector< std::vector< std::vector<double> > > F_old_global (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global    (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global   (num_step_space+1,Zmatrix );




    //===========================================================================================
    double a_cur = 2.93;
    double Inc_a_with_time = (a_cur - A_ref) / (time_loading/dt);

    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        std::vector<double> IntegralFunction (num_step_space+1, 0.0);
        double Area = 0.0;

        std::vector<double> Stress_radial(num_step_space+1, 0.0);
        std::vector<double> Stress_tan   (num_step_space+1, 0.0);
        std::vector<double> Stress_axial (num_step_space+1, 0.0);

        double a_int;
        if (time <= time_loading)
        {
            a_int = A_ref + Inc_a_with_time * step;
        }
        else
        {
            a_int = a_cur;
        }


        //=================================================================================================
        // space looping
        for (int loop2 = 0; loop2 <= num_step_space; loop2++)
        {
            double R_ref = A_ref + dR_ref*loop2;
            double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

            double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
            double lambda_T = r_int/R_ref;

            std::vector< std::vector<double> > F_tensor = Imatrix;
            F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


            std::vector< std::vector<double> > PassiveStress = Zmatrix;
            fnvObj.CalculateStress(F_tensor,F_old_global[loop2],Soe_global[loop2],Be_global[loop2],dt,
                                    PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


            // solve equilibrium equation
            IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
            double StripArea;
            if (R_ref == A_ref)
            {
                StripArea=0.0;
            }
            else
            {
                StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
            }
            Area = Area+StripArea;

            Stress_radial[loop2]= Area;
            Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
            Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

        }// end of space looping, i.e. loop2


        double Hyd_pressure = -1.0*Stress_radial[num_step_space];
        for (int i=0;i<=num_step_space;i++)
        {
            Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
            Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
            Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
        }


        if (time >= 0.0)
        {
            for (unsigned i=0;i<Stress_radial.size(); i++)
            {
                outputFile1 << Stress_radial[i] << "\t";
            }
            outputFile1 << std::endl;

//            for (unsigned i=0;i<Stress_tan.size(); i++)
//            {
//                outputFile2 << Stress_tan[i] << "\t";
//            }
//            outputFile2 << std::endl;
        }

    }// end of time looping, i.e. step


    // // // 2nd test
    a_cur = 1.5;
    Inc_a_with_time = (a_cur - A_ref) / (time_loading/dt);

    std::vector< std::vector< std::vector<double> > > F_old_global2 (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global2   (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global2   (num_step_space+1,Zmatrix );

    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        std::vector<double> IntegralFunction (num_step_space+1, 0.0);
        double Area = 0.0;

        std::vector<double> Stress_radial(num_step_space+1, 0.0);
        std::vector<double> Stress_tan   (num_step_space+1, 0.0);
        std::vector<double> Stress_axial (num_step_space+1, 0.0);

        double a_int;
        if (time <= time_loading)
        {
            a_int = A_ref + Inc_a_with_time * step;
        }
        else
        {
            a_int = a_cur;
        }


        //=================================================================================================
        // space looping
        for (int loop2 = 0; loop2 <= num_step_space; loop2++)
        {
            double R_ref = A_ref + dR_ref*loop2;
            double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

            double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
            double lambda_T = r_int/R_ref;

            std::vector< std::vector<double> > F_tensor = Imatrix;
            F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


            std::vector< std::vector<double> > PassiveStress = Zmatrix;
            fnvObj.CalculateStress(F_tensor,F_old_global2[loop2],Soe_global2[loop2],Be_global2[loop2],dt,
                                    PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


            // solve equilibrium equation
            IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
            double StripArea;
            if (R_ref == A_ref)
            {
                StripArea=0.0;
            }
            else
            {
                StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
            }
            Area = Area+StripArea;

            Stress_radial[loop2]= Area;
            Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
            Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

        }// end of space looping, i.e. loop2


        double Hyd_pressure = -1.0*Stress_radial[num_step_space];
        for (int i=0;i<=num_step_space;i++)
        {
            Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
            Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
            Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
        }


        if (time >= 0.0)
        {
            for (unsigned i=0;i<Stress_radial.size(); i++)
            {
                outputFile2 << Stress_radial[i] << "\t";
            }
            outputFile2 << std::endl;

//            for (unsigned i=0;i<Stress_tan.size(); i++)
//            {
//                outputFile2 << Stress_tan[i] << "\t";
//            }
//            outputFile2 << std::endl;
        }

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();
    outputFile2.close();

}



TEST(Carniel2015Creeping)
{
    std::cout << "start test: Carniel2015Creeping " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/Carniel2015Creeping", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/Carniel2015Creeping/Time_Pressure_aint.txt");


    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_loading    = 4.0;// time for stretching, unit: sec
    const double time_holding    = 6.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;


    double FinalInternalPressure = 1.86;




    //===========================================================================================
    // space discritization
    const double A_ref = 26.0;
    const double B_ref = 27.0;
    const double lambda_axialstretch = 1.0;
    const double dR_ref          = 0.1;// space discritization


    //===========================================================================================
    // material
    double C1e  = 0.29;
    double C2e  = 30.5;
    double C1oe = 0.5;
    double C2oe = C2e;
    double eta  = 839.4;


    // vector & tensor initialization ==================================================================================
    sneObj.SetGeometryParamsForGoldenSection(A_ref,B_ref,dR_ref,lambda_axialstretch);
    sneObj.SetMaterialParamsForGoldenSection(C1e,C2e,C1oe,C2oe,eta);


    //===========================================================================================
    // time looping
    double a_int = A_ref;

    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        double AppliedPressure;
        if (time <= time_loading)
        {
            AppliedPressure = FinalInternalPressure * ( time/time_loading);
        }
        else
        {
            AppliedPressure = FinalInternalPressure;
        }
        a_int = sneObj.FindRootGoldenSection(AppliedPressure,a_int,dt);

        outputFile1 << time << "\t" << AppliedPressure << "\t" << (a_int/A_ref) << std::endl;

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();

}

TEST(Carniel2015CreepingHyperelastic)
{
    std::cout << "start test: Carniel2015CreepingHyperelastic " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/Carniel2015Creeping", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/Carniel2015Creeping/Time_Pressure_aint_hyper.txt");


    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_loading    = 4.0;// time for stretching, unit: sec
    const double time_holding    = 6.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;


    double FinalInternalPressure = 1.86;




    //===========================================================================================
    // space discritization
    const double A_ref = 26.0;
    const double B_ref = 27.0;
    const double lambda_axialstretch = 1.0;
    const double dR_ref          = 0.1;// space discritization


    //===========================================================================================
    // material
    double C1e  = 1.2;
    double C2e  = 20.5;
    double C1oe = 0.0;
    double C2oe = C2e;
    double eta  = 839.4;


    // vector & tensor initialization ==================================================================================
    sneObj.SetGeometryParamsForGoldenSection(A_ref,B_ref,dR_ref,lambda_axialstretch);
    sneObj.SetMaterialParamsForGoldenSection(C1e,C2e,C1oe,C2oe,eta);


    //===========================================================================================
    // time looping
    double a_int = A_ref;

    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        double AppliedPressure;
        if (time <= time_loading)
        {
            AppliedPressure = FinalInternalPressure * ( time/time_loading);
        }
        else
        {
            AppliedPressure = FinalInternalPressure;
        }
        a_int = sneObj.FindRootGoldenSection(AppliedPressure,a_int,dt);

        outputFile1 << time << "\t" << AppliedPressure << "\t" << (a_int/A_ref) << std::endl;

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();

}

TEST(Carniel2015CreepingNeoHookean)
{
    std::cout << "start test: Carniel2015CreepingNeoHookean " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/Carniel2015Creeping", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/Carniel2015Creeping/Time_Pressure_aint_neo.txt");


    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_loading    = 4.0;// time for stretching, unit: sec
    const double time_holding    = 6.0;// time for holding constant stretch
    const double time_simulation =  time_loading+time_holding; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;


    double FinalInternalPressure = 1.86;




    //===========================================================================================
    // space discritization
    const double A_ref = 26.0;
    const double B_ref = 27.0;
    const double lambda_axialstretch = 1.0;
    const double dR_ref          = 0.1;// space discritization


    //===========================================================================================
    // material
    double Ce   = 52;
    double Coe  = 12;
    double eta  = 839.4;


    // vector & tensor initialization ==================================================================================
    sneObj.SetGeometryParamsForGoldenSection(A_ref,B_ref,dR_ref,lambda_axialstretch);
    sneObj.SetMaterialParamsForGoldenSection(Ce,Coe,eta);


    //===========================================================================================
    // time looping
    double a_int = A_ref;

    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        double AppliedPressure;
        if (time <= time_loading)
        {
            AppliedPressure = FinalInternalPressure * ( time/time_loading);
        }
        else
        {
            AppliedPressure = FinalInternalPressure;
        }
        a_int = sneObj.FindRootGoldenSection(AppliedPressure,a_int,dt);

        outputFile1 << time << "\t" << AppliedPressure << "\t" << (a_int/A_ref) << std::endl;

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();

}




TEST(ArhanStressRelaxation)
{
    std::cout << "start test: ArhanStressRelaxation " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/ArhanStressRelaxation", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/ArhanStressRelaxation/RadialStress.txt");
    std::ofstream outputFile2;
    outputFile2.open("./output/ArhanStressRelaxation/TangentialStress.txt");



    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_loading    = 0.1;// time for stretching, unit: sec
    const double time_holding    = 35.0;// time for holding constant stretch
    const double time_simulation = time_loading+time_holding; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;


    //===========================================================================================
    // space discritization
    const double A_ref = 1.1;
    const double B_ref = 1.4;
    const double a_cur = 2.16;
    const double lambda_axialstretch = 1.0;

    const double dR_ref          = 0.01;// space discritization
    const int    num_step_space  = (B_ref-A_ref)/dR_ref;
    const double Inc_a_with_time = (a_cur - A_ref) / (time_loading/dt);





    //===========================================================================================
    // material
    fnvObj.SetNeoHookeanMaterialParams(6.2,6.8,140.15);// healthy tissue
//    fnvObj.SetNeoHookeanMaterialParams(19.3,10.2,240.0);// diseased tissue




    // vector & tensor initialization ==================================================================================
    // initialization of tensors
    std::vector< std::vector<double> > Imatrix = Hobj.Identity(3);// Identity matrix
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix
    std::vector< std::vector< std::vector<double> > > F_old_global (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global    (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global   (num_step_space+1,Zmatrix );






    //===========================================================================================
    // time looping
    for (int step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        std::vector<double> IntegralFunction (num_step_space+1, 0.0);
        double Area = 0.0;

        std::vector<double> Stress_radial(num_step_space+1, 0.0);
        std::vector<double> Stress_tan   (num_step_space+1, 0.0);
        std::vector<double> Stress_axial (num_step_space+1, 0.0);

        double a_int;
        if (time <= time_loading)
        {
            a_int = A_ref + Inc_a_with_time * step;
        }
        else
        {
            a_int = a_cur;
        }


        //=================================================================================================
        // space looping
        for (int loop2 = 0; loop2 <= num_step_space; loop2++)
        {
            double R_ref = A_ref + dR_ref*loop2;
            double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

            double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
            double lambda_T = r_int/R_ref;

            std::vector< std::vector<double> > F_tensor = Imatrix;
            F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


            std::vector< std::vector<double> > PassiveStress = Zmatrix;
            fnvObj.CalculateStress(F_tensor,F_old_global[loop2],Soe_global[loop2],Be_global[loop2],dt,
                                    PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


            // solve equilibrium equation
            IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
            double StripArea;
            if (R_ref == A_ref)
            {
                StripArea=0.0;
            }
            else
            {
                StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
            }
            Area = Area+StripArea;

            Stress_radial[loop2]= Area;
            Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
            Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

        }// end of space looping, i.e. loop2


        double Hyd_pressure = -1.0*Stress_radial[num_step_space];
        for (int i=0;i<=num_step_space;i++)
        {
            Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
            Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
            Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
        }


        if (time >= 0.0)
        {
            for (unsigned i=0;i<Stress_radial.size(); i++)
            {
                outputFile1 << Stress_radial[i] << "\t";
            }
            outputFile1 << std::endl;

            for (unsigned i=0;i<Stress_tan.size(); i++)
            {
                outputFile2 << Stress_tan[i] << "\t";
            }
            outputFile2 << std::endl;
        }

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();

}

TEST(Liao2010Inflation)
{
    std::cout << "start test: Liao2010Inflation " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/Liao2010Inflation", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/Liao2010Inflation/Time_Pressure_aint.txt");
    std::ofstream outputFile2;
    outputFile2.open("./output/Liao2010Inflation/Radialstress.txt");
    std::ofstream outputFile3;
    outputFile3.open("./output/Liao2010Inflation/Circumstress.txt");


    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_simulation = 14.0; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;

    double FinalInternalPressure = 0.7;


    //===========================================================================================
    // space discritization
    const double A_ref = 1.6;
    const double B_ref = 3.0;
    const double lambda_axialstretch = 1.23;
    const double dR_ref          = 0.1;// space discritization
    const int    num_step_space  = (B_ref-A_ref)/dR_ref;

    //===========================================================================================
    // material
    double C1e  = 0.012;
    double C2e  = 2.8;
    double C1oe = 0.0;
    double C2oe = C2e;
    double eta  = 2.0;
    fnvObj.SetHumphreyMaterialParams(C1e,C2e,C1oe,C2oe,eta);


    // vector & tensor initialization ==================================================================================
    sneObj.SetGeometryParamsForGoldenSection(A_ref,B_ref,dR_ref,lambda_axialstretch);
    sneObj.SetMaterialParamsForGoldenSection(C1e,C2e,C1oe,C2oe,eta);

    // initialization of tensors
    std::vector< std::vector<double> > Imatrix = Hobj.Identity(3);// Identity matrix
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix
    std::vector< std::vector< std::vector<double> > > F_old_global (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global    (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global   (num_step_space+1,Zmatrix );


    //===========================================================================================
    // time looping
    double a_int = 1.4;

    for (unsigned step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        double AppliedPressure= FinalInternalPressure * ( time/time_simulation);

        a_int = sneObj.FindRootGoldenSection(AppliedPressure,a_int,dt);

        double b_int = sqrt(a_int*a_int + (1.0/lambda_axialstretch)*(B_ref*B_ref-A_ref*A_ref));

        outputFile1 << time << "\t" << AppliedPressure << "\t" << b_int << std::endl;


        if (false)
        {
            std::vector<double> IntegralFunction (num_step_space+1, 0.0);
            double Area = 0.0;

            std::vector<double> Stress_radial(num_step_space+1, 0.0);
            std::vector<double> Stress_tan   (num_step_space+1, 0.0);
            std::vector<double> Stress_axial (num_step_space+1, 0.0);

             //=================================================================================================
            // space looping
            for (int loop2 = 0; loop2 <= num_step_space; loop2++)
            {
                double R_ref = A_ref + dR_ref*loop2;
                double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

                double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
                double lambda_T = r_int/R_ref;

                std::vector< std::vector<double> > F_tensor = Imatrix;
                F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


                std::vector< std::vector<double> > PassiveStress =Zmatrix;
                fnvObj.CalculateStress(F_tensor,F_old_global[loop2],Soe_global[loop2],Be_global[loop2],dt,
                                        PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


                // solve equilibrium equation
                IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
                double StripArea;
                if (R_ref == A_ref)
                {
                    StripArea=0.0;
                }
                else
                {
                    StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
                }
                Area = Area+StripArea;

                Stress_radial[loop2]= Area;
                Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
                Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

            }// end of space looping, i.e. loop2


            double Hyd_pressure = -1.0*Stress_radial[num_step_space];
            for (int i=0;i<=num_step_space;i++)
            {
                Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
                Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
                Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
            }

            assert(isnan(Stress_radial[0]) == false);

            for (unsigned i=0;i<Stress_radial.size(); i++)
            {
                outputFile2 << Stress_radial[i] << "\t";
                outputFile3 << Stress_tan[i] << "\t";
            }
            outputFile2 << std::endl;
            outputFile3 << std::endl;
        }

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();
    outputFile2.close();
    outputFile3.close();

}



TEST(ViscoelasticCylinderInflation)
{
    std::cout << "start test: ViscoelasticCylinderInflation " <<std::endl;
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;
    SolveNonlinearEquation sneObj;


    //===========================================================================================
    // File I/O
    mkdir("./output/ViscoelasticCylinderInflation", S_IRWXU);

    std::ofstream outputFile1;
    outputFile1.open("./output/ViscoelasticCylinderInflation/Time_Pressure_aint.txt");
    std::ofstream outputFile2;
    outputFile2.open("./output/ViscoelasticCylinderInflation/Radialstress.txt");
    std::ofstream outputFile3;
    outputFile3.open("./output/ViscoelasticCylinderInflation/Circumstress.txt");


    //===========================================================================================
    // time discritization
    const double dt              = 0.01;// time increment for mechanics, unit: sec
    const double time_simulation = 80.0; // total simulation time, unit: sec
    const int    num_step_time   = time_simulation / dt;

    unsigned PrintFreq = 80;

    double FinalInternalPressure = 20.0;


    //===========================================================================================
    // space discritization
    const double A_ref = 0.35;
    const double B_ref = 1.10;
    const double lambda_axialstretch = 1.0;
    const double dR_ref          = 0.1;// space discritization
    const int    num_step_space  = (B_ref-A_ref)/dR_ref;

    //===========================================================================================
    // material
    double Ce  = 10.0;
    double Coe = 50.0;
    double eta = 300.0;
    fnvObj.SetNeoHookeanMaterialParams(Ce,Coe,eta);


    // vector & tensor initialization ==================================================================================
    sneObj.SetGeometryParamsForGoldenSection(A_ref,B_ref,dR_ref,lambda_axialstretch);
    sneObj.SetMaterialParamsForGoldenSection(Ce,Coe,eta);

    // initialization of tensors
    std::vector< std::vector<double> > Imatrix = Hobj.Identity(3);// Identity matrix
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix
    std::vector< std::vector< std::vector<double> > > F_old_global (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Be_global    (num_step_space+1,Imatrix );
    std::vector< std::vector< std::vector<double> > > Soe_global   (num_step_space+1,Zmatrix );


    //===========================================================================================
    // time looping
    double a_int = A_ref;

    for (unsigned step=0; step<=num_step_time; step++)
    {
        double time = dt*step;// time for mechanics calculation,  unit: sec

        double AppliedPressure= FinalInternalPressure * ( time/time_simulation);

        a_int = sneObj.FindRootGoldenSection(AppliedPressure,a_int,dt);

        outputFile1 << time << "\t" << AppliedPressure << "\t" << a_int << std::endl;


        if (true)
        {
            std::vector<double> IntegralFunction (num_step_space+1, 0.0);
            double Area = 0.0;

            std::vector<double> Stress_radial(num_step_space+1, 0.0);
            std::vector<double> Stress_tan   (num_step_space+1, 0.0);
            std::vector<double> Stress_axial (num_step_space+1, 0.0);

             //=================================================================================================
            // space looping
            for (int loop2 = 0; loop2 <= num_step_space; loop2++)
            {
                double R_ref = A_ref + dR_ref*loop2;
                double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

                double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
                double lambda_T = r_int/R_ref;

                std::vector< std::vector<double> > F_tensor = Imatrix;
                F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


                std::vector< std::vector<double> > PassiveStress =Zmatrix;
                fnvObj.CalculateStress(F_tensor,F_old_global[loop2],Soe_global[loop2],Be_global[loop2],dt,
                                        PassiveStress,fnvObj.CauchyStress,fnvObj.MultiaxialLoading);


                // solve equilibrium equation
                IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
                double StripArea;
                if (R_ref == A_ref)
                {
                    StripArea=0.0;
                }
                else
                {
                    StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
                }
                Area = Area+StripArea;

                Stress_radial[loop2]= Area;
                Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
                Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

            }// end of space looping, i.e. loop2


            double Hyd_pressure = -1.0*Stress_radial[num_step_space];
            for (int i=0;i<=num_step_space;i++)
            {
                Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
                Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
                Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
            }

            assert(isnan(Stress_radial[0]) == false);

            if ( (step % PrintFreq) == 0 )
            {
                for (unsigned i=0;i<Stress_radial.size(); i++)
                {
                    outputFile2 << Stress_radial[i] << "\t";
                    outputFile3 << Stress_tan[i] << "\t";
                }
                outputFile2 << std::endl;
                outputFile3 << std::endl;
            }

        }

    }// end of time looping, i.e. step


    //===========================================================================================
    // cleaning
    outputFile1.close();
    outputFile2.close();
    outputFile3.close();

}





} // end of namespace




