#include "SolveNonlinearEquation.hpp"

SolveNonlinearEquation::SolveNonlinearEquation()
{
    IsInitialGuessPresent = false;
    FunEvaTol = 1e-6;
    MaxFunEvaNumber = 1e3;

    IsGoldenSectionParamsPresent = false;
}



void SolveNonlinearEquation::SetParameterValues(double rCe, double rCoe, double rlambda, double rlambda_e, double rdt, double reta, double rStress)
{
    Ce = rCe;
    Coe = rCoe;
    lambda = rlambda;
    lambda_e = rlambda_e;
    dt = rdt;
    eta = reta;
    Stress = rStress;
}


void SolveNonlinearEquation::SetInitialGuess(double rInitialGuess)
{
    mInitialGuess = rInitialGuess;
    IsInitialGuessPresent = true;
}



double SolveNonlinearEquation::SolveEquation()
{
    if (IsInitialGuessPresent == false )
    {
        std::cerr << "Error: No initial guess has been provided !!! "  << std::endl;
        exit(-1);
    }

    double ErrorValue = 1.0;
    double Solution = mInitialGuess;
    int FunEvaNumber =1;

    while (ErrorValue > FunEvaTol)
    {
        double FunValue = CalculateFunctionValue(Solution);
        double Gradient = CalculateGradientValue(Solution);

        double NewSolution = Solution - (FunValue/Gradient);
        ErrorValue = fabs(NewSolution-Solution);

        if (FunEvaNumber == MaxFunEvaNumber)
        {
            break;
        }
        FunEvaNumber ++;

        Solution = NewSolution;
    }

    return Solution;
}



double SolveNonlinearEquation::CalculateFunctionValue(double rGuess)
{
    double FunValue = 2.0*Ce*(rGuess*rGuess-(1.0/rGuess))
                + 2.0*Coe*( 2.0*((rGuess*lambda_e*lambda_e/(lambda))-(sqrt(lambda)/(sqrt(rGuess)*lambda_e)))
                - (lambda_e*lambda_e - (1.0/lambda_e))
                - ((4.0*dt*Coe)/(3.0*eta))*(2.0*pow(lambda_e,4.0) - lambda_e - (1.0/(lambda_e*lambda_e))) )
                - Stress;

    return FunValue;
}

double SolveNonlinearEquation::CalculateGradientValue(double rGuess)
{
    double GradValue = 2.0*Ce*(2.0*rGuess+(1.0/(rGuess*rGuess)))
        + 4.0*Coe*((lambda_e*lambda_e/lambda)+(0.5*(sqrt(lambda)/(sqrt(rGuess*rGuess*rGuess)*lambda_e))));

    return GradValue;
}






double SolveNonlinearEquation::FindRootGoldenSection(double DesiredPressure, double Guess, double rdt)
{
    if (IsGoldenSectionParamsPresent == false)
    {
        std::cerr << "Error: Golden section method parameters have not been provided !!! "  << std::endl;
        exit(-1);
    }


    double gold_r = (sqrt(5) - 1.0) / 2.0;
    double SpaceIncrement = 0.1;
    double point_a = Guess;
    double point_b = Guess+SpaceIncrement;

    double point_x1 = (point_b - point_a)*(1.0-gold_r) + point_a;
    double point_x2 = (point_b - point_a)*gold_r + point_a;

    std::vector< std::vector< std::vector<double> > > temp_F_old = F_old_global;
    std::vector< std::vector< std::vector<double> > > temp_Be = Be_global;
    std::vector< std::vector< std::vector<double> > > temp_Soe = Soe_global;
    double fx1 = fabs(DesiredPressure - CalculateObjectiveFunction(point_x1,rdt,temp_F_old,temp_Be,temp_Soe,false));
    double fx2 = fabs(DesiredPressure - CalculateObjectiveFunction(point_x2,rdt,temp_F_old,temp_Be,temp_Soe,false));

    for (int i =0; i< MaxFunEvaNumber; i++ )
    {
        if (fx1 >= fx2)
        {
            point_a = point_x1;
            point_x1 = point_x2;
            point_x2 = (point_b - point_a)*gold_r + point_a;
            fx1 = fx2;
            fx2 = fabs(DesiredPressure - CalculateObjectiveFunction(point_x2,rdt,temp_F_old,temp_Be,temp_Soe,false));
        }
        else
        {
            point_b = point_x2;
            point_x2 = point_x1;
            point_x1 = (point_b - point_a)*(1.0-gold_r) + point_a;
            fx2 = fx1;
            fx1 = fabs(DesiredPressure - CalculateObjectiveFunction(point_x1,rdt,temp_F_old,temp_Be,temp_Soe,false));
        }

        if (fabs(point_a-point_b) < FunEvaTol)
        {
            break;
        }
    }

    double Pressure = CalculateObjectiveFunction(point_a,rdt,temp_F_old,temp_Be,temp_Soe,true);
    assert(fabs(Pressure-DesiredPressure) < 1e-3);

    F_old_global = temp_F_old;
    Be_global = temp_Be;
    Soe_global = temp_Soe;

    return point_a;

}


double SolveNonlinearEquation::CalculateObjectiveFunction(double a_int, double rdt,
                                            std::vector< std::vector< std::vector<double> > > & F_old,
                                            std::vector< std::vector< std::vector<double> > > & Be,
                                            std::vector< std::vector< std::vector<double> > > & Soe,
                                            bool RewriteTensors)
{
    //===========================================================================================
    // Class initialisation
    HelperFunction Hobj;
    FNVModel fnvObj;


    //===========================================================================================
    // material
    if (rMaterialType == NeoHookean)
    {
        fnvObj.SetNeoHookeanMaterialParams(Ce_NH,Coe_NH,eta_NH);
    }
    else if (rMaterialType == Humphrey)
    {
        fnvObj.SetHumphreyMaterialParams(C1e_HM,C2e_HM,C1oe_HM,C2oe_HM,eta_HM);
    }
    else
    {
        std::cerr << "Error: No material law has been provided for Golden section method !!! "  << std::endl;
        exit(-1);
    }



     //=================================================================================================
    // space looping
    std::vector<double> IntegralFunction (num_step_space+1, 0.0);
    double Area = 0.0;

    std::vector<double> Stress_radial(num_step_space+1, 0.0);
    std::vector<double> Stress_tan   (num_step_space+1, 0.0);
    std::vector<double> Stress_axial (num_step_space+1, 0.0);


    std::vector< std::vector<double> > Imatrix (3,std::vector<double> (3,0.0) );// zero matrix
    Imatrix[0][0]=1.0;Imatrix[1][1]=1.0;Imatrix[2][2]=1.0;
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix


     for (int loop2 = 0; loop2 <= num_step_space; loop2++)
    {
        double R_ref = A_ref + dR_ref*loop2;
        double r_int = sqrt(a_int*a_int + (1.0/(lambda_axialstretch))* (R_ref*R_ref - A_ref*A_ref));

        double lambda_R = (1.0/(lambda_axialstretch))*(R_ref/r_int);
        double lambda_T = r_int/R_ref;

        std::vector< std::vector<double> > F_tensor = Imatrix;
        F_tensor[0][0] = lambda_R; F_tensor[1][1]=lambda_T; F_tensor[2][2] = lambda_axialstretch;


        std::vector< std::vector<double> > PassiveStress = Zmatrix;

        if (RewriteTensors == true)
        {
            fnvObj.CalculateStress(F_tensor,F_old[loop2],Soe[loop2],Be[loop2],rdt,PassiveStress,
                                    fnvObj.CauchyStress,fnvObj.MultiaxialLoading);
        }
        else
        {
            std::vector< std::vector<double> > temp_F_old =F_old[loop2];
            std::vector< std::vector<double> > temp_Soe = Soe[loop2];
            std::vector< std::vector<double> > temp_Be = Be[loop2];
            fnvObj.CalculateStress(F_tensor,temp_F_old,temp_Soe,temp_Be,rdt,PassiveStress,
                                        fnvObj.CauchyStress,fnvObj.MultiaxialLoading);
        }


        // solve equilibrium equation
        IntegralFunction[loop2] = (1.0/(lambda_axialstretch))*((PassiveStress[1][1]-PassiveStress[0][0])/r_int)*(R_ref/r_int);
        double StripArea;
        if (R_ref == A_ref)
        {
            StripArea=0.0;
        }
        else
        {
            StripArea =  (dR_ref/2.0)*(IntegralFunction[loop2]+IntegralFunction[loop2-1]);
        }
        Area = Area+StripArea;

        Stress_radial[loop2]= Area;
        Stress_tan[loop2] = Area+(PassiveStress[1][1] - PassiveStress[0][0]);
        Stress_axial[loop2]=Area+(PassiveStress[2][2] - PassiveStress[0][0]);

    }// end of space looping, i.e. loop2


    double Hyd_pressure = -1.0*Stress_radial[num_step_space];
    for (int i=0;i<=num_step_space;i++)
    {
        Stress_radial[i] = Stress_radial[i]+Hyd_pressure;
        Stress_tan[i] =  Stress_tan[i]+Hyd_pressure;
        Stress_axial[i] = Stress_axial[i]+Hyd_pressure;
    }


    double Pressure = -Stress_radial[0];
    assert(isnan(Pressure) == false);
    return Pressure;

}


void SolveNonlinearEquation::SetGeometryParamsForGoldenSection(double rA_ref, double rB_ref, double rdR_ref, double rlambda_axialstretch )
{
    IsGoldenSectionParamsPresent = true;


    A_ref = rA_ref;
    B_ref = rB_ref;
    lambda_axialstretch = rlambda_axialstretch;
    dR_ref = rdR_ref;// space discritization
    num_step_space  = (B_ref-A_ref)/dR_ref;

    std::vector< std::vector<double> > Imatrix (3,std::vector<double> (3,0.0) );// zero matrix
    Imatrix[0][0]=1.0;Imatrix[1][1]=1.0;Imatrix[2][2]=1.0;
    std::vector< std::vector<double> > Zmatrix (3,std::vector<double> (3,0.0) );// zero matrix

    F_old_global.resize(num_step_space+1,Imatrix) ;
    Be_global.resize(num_step_space+1,Imatrix) ;
    Soe_global.resize(num_step_space+1,Zmatrix);

}


void SolveNonlinearEquation::SetMaterialParamsForGoldenSection(double rCe_NH, double rCoe_NH, double reta_NH)
{
    Ce_NH = rCe_NH;
    Coe_NH = rCoe_NH;
    eta_NH = reta_NH;

    rMaterialType = NeoHookean;
}


void SolveNonlinearEquation::SetMaterialParamsForGoldenSection(double rC1e_HM, double rC2e_HM, double rC1oe_HM, double rC2oe_HM, double reta_HM)
{
    C1e_HM = rC1e_HM;
    C2e_HM = rC2e_HM;
    C1oe_HM = rC1oe_HM;
    C2oe_HM = rC2oe_HM;
    eta_HM = reta_HM;

    rMaterialType = Humphrey;
}











